@@ -10,7 +10,11 @@
 #undef HAVE_PL_PERL
 #undef HAVE_STATIC_PERL
 #undef HAVE_GMODULE
+#undef HAVE_GC_H
+#undef HAVE_GC_GC_H
 #undef WANT_BIG5
+#undef USE_GC
+#undef HAVE_GLIB2
 
 /* macros/curses checks */
 #undef HAS_CURSES
@@ -37,5 +41,13 @@
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
 
-/* SSL */
-#undef HAVE_OPENSSL
+/* If set to 64, enables 64bit off_t for some systems (eg. Linux, Solaris) */
+#undef _FILE_OFFSET_BITS
+
+/* What type should be used for uoff_t */
+#undef UOFF_T_INT
+#undef UOFF_T_LONG
+#undef UOFF_T_LONG_LONG
+
+/* printf()-format for uoff_t, eg. "u" or "lu" or "llu" */
+#undef PRIuUOFF_T
