@@ -27,6 +27,7 @@
 #include "settings.h"
 
 #include "themes.h"
+#include "fe-irc-server.h"
 
 void fe_irc_modules_init(void);
 void fe_irc_modules_deinit(void);
@@ -37,6 +38,9 @@ void fe_irc_channels_deinit(void);
 void fe_irc_queries_init(void);
 void fe_irc_queries_deinit(void);
 
+void fe_irc_layout_init(void);
+void fe_irc_layout_deinit(void);
+
 void fe_irc_messages_init(void);
 void fe_irc_messages_deinit(void);
 
@@ -46,9 +50,6 @@ void fe_irc_commands_deinit(void);
 void fe_ircnet_init(void);
 void fe_ircnet_deinit(void);
 
-void fe_irc_server_init(void);
-void fe_irc_server_deinit(void);
-
 void fe_ctcp_init(void);
 void fe_ctcp_deinit(void);
 
@@ -67,6 +68,12 @@ void fe_netsplit_deinit(void);
 void fe_netjoin_init(void);
 void fe_netjoin_deinit(void);
 
+void fe_whois_init(void);
+void fe_whois_deinit(void);
+
+void irc_completion_init(void);
+void irc_completion_deinit(void);
+
 void fe_common_irc_init(void)
 {
 	settings_add_bool("lookandfeel", "show_away_once", TRUE);
@@ -85,6 +92,8 @@ void fe_common_irc_init(void)
 	fe_modes_init();
 	fe_netsplit_init();
 	fe_netjoin_init();
+        fe_whois_init();
+        irc_completion_init();
 
 	settings_check();
 	module_register("core", "fe-irc");
@@ -108,6 +117,8 @@ void fe_common_irc_deinit(void)
 	fe_modes_deinit();
 	fe_netsplit_deinit();
 	fe_netjoin_deinit();
+        fe_whois_deinit();
+        irc_completion_deinit();
 
 	theme_unregister();
 }
