@@ -184,8 +184,7 @@ static void print_netjoins(NETJOIN_SERVER_REC *server)
 		next = tmp->next;
 		while (rec->now_channels != NULL) {
 			char *channel = rec->now_channels->data;
-			char *realchannel = channel +
-				(isnickflag(*channel) && ischannel(channel[1]));
+			char *realchannel = channel + 1;
 
 			temp = g_hash_table_lookup(channels, realchannel);
 			if (temp == NULL) {
@@ -198,7 +197,7 @@ static void print_netjoins(NETJOIN_SERVER_REC *server)
 
 			temp->count++;
 			if (temp->count <= netjoin_max_nicks) {
-				if (isnickflag(*channel))
+				if (*channel != ' ')
 					g_string_append_c(temp->nicks,
 							  *channel);
 				g_string_sprintfa(temp->nicks, "%s, ",
@@ -209,15 +208,15 @@ static void print_netjoins(NETJOIN_SERVER_REC *server)
 			old = gslist_find_icase_string(rec->old_channels,
 						       realchannel);
 			if (old != NULL) {
-				g_free(old->data);
+				void *data = old->data;
 				rec->old_channels =
-					g_slist_remove(rec->old_channels,
-						       old->data);
+					g_slist_remove(rec->old_channels, data);
+				g_free(data);
 			}
 
-			g_free(channel);
 			rec->now_channels =
 				g_slist_remove(rec->now_channels, channel);
+			g_free(channel);
 		}
 
 		if (rec->old_channels == NULL)
@@ -318,35 +317,47 @@ static void msg_join(IRC_SERVER_REC *server, const char *channel,
 		netjoin = netjoin_add(server, nick, split->channels);
 
 	netjoin->now_channels = g_slist_append(netjoin->now_channels,
-					       g_strdup(channel));
+					       g_strconcat(" ", channel, NULL));
 	signal_stop();
 }
 
-static int netjoin_set_nickmode(NETJOIN_REC *rec, const char *channel,
-				char mode)
+static int netjoin_set_nickmode(IRC_SERVER_REC *server, NETJOIN_REC *rec,
+				const char *channel, char prefix)
 {
 	GSList *pos;
+	const char *flags;
+	char *found_chan = NULL;
 
-	pos = gslist_find_icase_string(rec->now_channels, channel);
-	if (pos == NULL)
+	for (pos = rec->now_channels; pos != NULL; pos = pos->next) {
+		char *chan = pos->data;
+		if (strcasecmp(chan+1, channel) == 0) {
+			found_chan = chan;
+			break;
+		}
+	}
+
+	if (found_chan == NULL)
 		return FALSE;
 
-	g_free(pos->data);
-	pos->data = g_strdup_printf("%c%s", mode, channel);
+	flags = server->get_nick_flags(SERVER(server));
+	while (*flags != '\0') {
+		if (found_chan[0] == *flags)
+			break;
+		if (prefix == *flags) {
+			found_chan[0] = prefix;
+			break;
+		}
+		flags++;
+	}
 	return TRUE;
 }
 
-#define isnickmode(c) \
-	((c) == 'o' || (c) == 'v' || (c) == 'h')
-#define nickmodechar(c) \
-	((c) == 'o' ? '@' : ((c) == 'v' ? '+' : ((c) == 'h' ? '%' : '\0')))
-
 static void msg_mode(IRC_SERVER_REC *server, const char *channel,
 		     const char *sender, const char *addr, const char *data)
 {
 	NETJOIN_REC *rec;
 	char *params, *mode, *nicks;
-	char **nicklist, **nick, type, modechr;
+	char **nicklist, **nick, type, prefix;
 	int show;
 
 	g_return_if_fail(data != NULL);
@@ -367,16 +378,16 @@ static void msg_mode(IRC_SERVER_REC *server, const char *channel,
 			continue;
 		}
 
-		if (*nick != NULL && isnickmode(*mode)) {
+		if (*nick != NULL && GET_MODE_PREFIX(server, *mode)) {
                         /* give/remove ops */
 			rec = netjoin_find(server, *nick);
-                        modechr = nickmodechar(*mode);
-			if (rec == NULL || type != '+' || modechr == '\0' ||
-			    !netjoin_set_nickmode(rec, channel, modechr))
+			prefix = GET_MODE_PREFIX(server, *mode);
+			if (rec == NULL || type != '+' || prefix == '\0' ||
+			    !netjoin_set_nickmode(server, rec, channel, prefix))
 				show = TRUE;
                         nick++;
 		} else {
-			if (HAS_MODE_ARG(type, *mode) && *nick != NULL)
+			if (HAS_MODE_ARG(server, type, *mode) && *nick != NULL)
 				nick++;
 			show = TRUE;
 		}
