@@ -74,11 +74,32 @@ char *irc_nick_strip(const char *nick)
 	return stripped;
 }
 
+int irc_nickcmp_rfc1459(const char *m, const char *n)
+{
+	while (*m != '\0' && *n != '\0') {
+		if (to_rfc1459(*m) != to_rfc1459(*n))
+			return -1;
+		m++; n++;
+	}
+	return *m == *n ? 0 : 1;
+}
+
+int irc_nickcmp_ascii(const char *m, const char *n)
+{
+	while (*m != '\0' && *n != '\0') {
+		if (to_ascii(*m) != to_ascii(*n))
+			return -1;
+		m++; n++;
+	}
+	return *m == *n ? 0 : 1;
+}
+
 static void event_names_list(IRC_SERVER_REC *server, const char *data)
 {
 	IRC_CHANNEL_REC *chanrec;
+	NICK_REC *rec;
 	char *params, *type, *channel, *names, *ptr;
-        int op, halfop, voice;
+        int op, halfop, voice, other;
 
 	g_return_if_fail(data != NULL);
 
@@ -117,8 +138,8 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 		   showing "@+nick" and since none of these chars are valid
 		   nick chars, just check them until a non-nickflag char is
 		   found. FIXME: we just ignore owner char now. */
-		op = halfop = voice = FALSE;
-		while (isnickflag(*ptr)) {
+		op = halfop = voice = other = FALSE;
+		while (isnickflag(server, *ptr)) {
 			switch (*ptr) {
 			case '@':
                                 op = TRUE;
@@ -129,11 +150,18 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 			case '+':
                                 voice = TRUE;
                                 break;
+			default:
+				other = *ptr;
 			}
                         ptr++;
 		}
 
-		irc_nicklist_insert(chanrec, ptr, op, halfop, voice, FALSE);
+		if (nicklist_find((CHANNEL_REC *) chanrec, ptr) == NULL) {
+			rec = irc_nicklist_insert(chanrec, ptr, op, halfop,
+						  voice, FALSE);
+			if (other)
+				rec->other = other;
+		}
 	}
 
 	g_free(params);
@@ -249,6 +277,30 @@ static void event_whois_away(SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void event_own_away(SERVER_REC *server, const char *data)
+{
+	char *params, *nick;
+
+	g_return_if_fail(data != NULL);
+
+	/* set user's gone flag.. */
+	params = event_get_params(data, 2, &nick, NULL);
+	nicklist_update_flags(server, nick, TRUE, -1);
+	g_free(params);
+}
+
+static void event_own_unaway(SERVER_REC *server, const char *data)
+{
+	char *params, *nick;
+
+	g_return_if_fail(data != NULL);
+
+	/* set user's gone flag.. */
+	params = event_get_params(data, 2, &nick, NULL);
+	nicklist_update_flags(server, nick, FALSE, -1);
+	g_free(params);
+}
+
 static void event_whois_ircop(SERVER_REC *server, const char *data)
 {
 	char *params, *nick, *awaymsg;
@@ -263,7 +315,7 @@ static void event_whois_ircop(SERVER_REC *server, const char *data)
 
 static void event_nick_in_use(IRC_SERVER_REC *server, const char *data)
 {
-	char *str;
+	char *str, *cmd;
 	int n;
 
 	g_return_if_fail(data != NULL);
@@ -302,7 +354,9 @@ static void event_nick_in_use(IRC_SERVER_REC *server, const char *data)
 		}
 	}
 
-	irc_send_cmdv(server, "NICK %s", server->nick);
+	cmd = g_strdup_printf("NICK %s", server->nick);
+	irc_send_cmd_now(server, cmd);
+	g_free(cmd);	
 }
 
 static void event_target_unavailable(IRC_SERVER_REC *server, const char *data)
@@ -326,6 +380,7 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 	char *params, *nick;
 
 	g_return_if_fail(data != NULL);
+	g_return_if_fail(orignick != NULL);
 
 	params = event_get_params(data, 1, &nick);
 
@@ -373,17 +428,20 @@ static void sig_usermode(SERVER_REC *server)
 	nicklist_update_flags(server, server->nick, server->usermode_away, -1);
 }
 
-static const char *get_nick_flags(void)
+static const char *get_nick_flags(SERVER_REC *server)
 {
-        return "@+%";
+	IRC_SERVER_REC *irc_server = (IRC_SERVER_REC *) server;
+	const char *prefix =
+		g_hash_table_lookup(irc_server->isupport, "PREFIX");
+
+	prefix = prefix == NULL ? NULL : strchr(prefix, ')');
+	return prefix == NULL ? "" : prefix+1;
 }
 
 static void sig_connected(IRC_SERVER_REC *server)
 {
-	if (IS_IRC_SERVER(server)) {
-		server->get_nick_flags =
-			(const char *(*)(void)) get_nick_flags;
-	}
+	if (IS_IRC_SERVER(server))
+		server->get_nick_flags = get_nick_flags;
 }
 
 void irc_nicklist_init(void)
@@ -393,10 +451,13 @@ void irc_nicklist_init(void)
 	signal_add("silent event who", (SIGNAL_FUNC) event_who);
 	signal_add("silent event whois", (SIGNAL_FUNC) event_whois);
 	signal_add_first("event 311", (SIGNAL_FUNC) event_whois);
-	signal_add_first("event 301", (SIGNAL_FUNC) event_whois_away);
-	signal_add_first("event 313", (SIGNAL_FUNC) event_whois_ircop);
+	signal_add_first("whois away", (SIGNAL_FUNC) event_whois_away);
+	signal_add_first("whois oper", (SIGNAL_FUNC) event_whois_ircop);
+	signal_add_first("event 306", (SIGNAL_FUNC) event_own_away);
+	signal_add_first("event 305", (SIGNAL_FUNC) event_own_unaway);
 	signal_add_first("event 353", (SIGNAL_FUNC) event_names_list);
 	signal_add_first("event 366", (SIGNAL_FUNC) event_end_of_names);
+	signal_add_first("event 432", (SIGNAL_FUNC) event_nick_in_use);
 	signal_add_first("event 433", (SIGNAL_FUNC) event_nick_in_use);
 	signal_add_first("event 437", (SIGNAL_FUNC) event_target_unavailable);
 	signal_add_first("event 302", (SIGNAL_FUNC) event_userhost);
@@ -412,10 +473,13 @@ void irc_nicklist_deinit(void)
 	signal_remove("silent event who", (SIGNAL_FUNC) event_who);
 	signal_remove("silent event whois", (SIGNAL_FUNC) event_whois);
 	signal_remove("event 311", (SIGNAL_FUNC) event_whois);
-	signal_remove("event 301", (SIGNAL_FUNC) event_whois_away);
-	signal_remove("event 313", (SIGNAL_FUNC) event_whois_ircop);
+	signal_remove("whois away", (SIGNAL_FUNC) event_whois_away);
+	signal_remove("whois oper", (SIGNAL_FUNC) event_whois_ircop);
+	signal_remove("event 306", (SIGNAL_FUNC) event_own_away);
+	signal_remove("event 305", (SIGNAL_FUNC) event_own_unaway);
 	signal_remove("event 353", (SIGNAL_FUNC) event_names_list);
 	signal_remove("event 366", (SIGNAL_FUNC) event_end_of_names);
+	signal_remove("event 432", (SIGNAL_FUNC) event_nick_in_use);
 	signal_remove("event 433", (SIGNAL_FUNC) event_nick_in_use);
 	signal_remove("event 437", (SIGNAL_FUNC) event_target_unavailable);
 	signal_remove("event 302", (SIGNAL_FUNC) event_userhost);
