@@ -45,7 +45,9 @@ static void sig_server_connect_copy(SERVER_CONNECT_REC **dest,
 	rec->max_kicks = src->max_kicks;
 	rec->max_modes = src->max_modes;
 	rec->max_msgs = src->max_msgs;
+	rec->max_whois = src->max_whois;
 	rec->usermode = g_strdup(src->usermode);
+	rec->alternate_nick = g_strdup(src->alternate_nick);
 	*dest = (SERVER_CONNECT_REC *) rec;
 }
 
@@ -68,8 +70,12 @@ static void sig_connected(IRC_SERVER_REC *server)
 	if (!IS_IRC_SERVER(server) || !server->connrec->reconnection)
 		return;
 
-	if (server->connrec->away_reason != NULL)
+	if (server->connrec->away_reason != NULL) {
 		irc_send_cmdv(server, "AWAY :%s", server->connrec->away_reason);
+
+		g_free(server->away_reason);
+		server->away_reason = g_strdup(server->connrec->away_reason);
+	}
 }
 
 static void event_nick_collision(IRC_SERVER_REC *server, const char *data)
@@ -84,7 +90,7 @@ static void event_nick_collision(IRC_SERVER_REC *server, const char *data)
 	   don't connect to the server more than once in every 10 seconds. */
 
 	new_connect = server->connect_time+10 -
-		settings_get_int("server_reconnect_time");
+		settings_get_time("server_reconnect_time")/1000;
 	if (server->connect_time > new_connect)
 		server->connect_time = new_connect;
 
