@@ -24,14 +24,15 @@
 #include "network.h"
 #include "misc.h"
 #include "settings.h"
-
+#include "net-sendbuffer.h"
 #include "irc-servers.h"
 
 #include "dcc-get.h"
+#include "dcc-send.h"
 
 static int dcc_file_create_mode;
 
-static GET_DCC_REC *dcc_get_create(IRC_SERVER_REC *server, CHAT_DCC_REC *chat,
+GET_DCC_REC *dcc_get_create(IRC_SERVER_REC *server, CHAT_DCC_REC *chat,
 				   const char *nick, const char *arg)
 {
 	GET_DCC_REC *dcc;
@@ -89,7 +90,7 @@ void dcc_get_send_received(GET_DCC_REC *dcc)
 {
 	guint32 recd;
 
-	recd = (guint32) htonl(dcc->transfd);
+	recd = (guint32) htonl(dcc->transfd & 0xffffffff);
 	memcpy(dcc->count_buf, &recd, 4);
 
 	dcc->count_pos =
@@ -133,7 +134,7 @@ static void sig_dccget_send(GET_DCC_REC *dcc)
 	}
 
 	memcpy(&recd, dcc->count_buf, 4);
-	if (recd != (guint32) htonl(dcc->transfd))
+	if (recd != (guint32) htonl(dcc->transfd & 0xffffffff))
                 dcc_get_send_received(dcc);
 }
 
@@ -172,21 +173,23 @@ static void sig_dccget_receive(GET_DCC_REC *dcc)
 }
 
 /* callback: net_connect() finished for DCC GET */
-static void sig_dccget_connected(GET_DCC_REC *dcc)
+void sig_dccget_connected(GET_DCC_REC *dcc)
 {
 	struct stat statbuf;
-	char *fname, *tempfname;
+	char *fname, *tempfname, *str;
         int ret, ret_errno, temphandle, old_umask;
 
-	if (net_geterror(dcc->handle) != 0) {
-		/* error connecting */
-		signal_emit("dcc error connect", 1, dcc);
-		dcc_destroy(DCC(dcc));
-		return;
-	}
+	if (!dcc->from_dccserver) {
+		if (net_geterror(dcc->handle) != 0) {
+			/* error connecting */
+			signal_emit("dcc error connect", 1, dcc);
+			dcc_destroy(DCC(dcc));
+			return;
+		}
 
-	g_source_remove(dcc->tagconn);
-        dcc->tagconn = -1;
+		g_source_remove(dcc->tagconn);
+		dcc->tagconn = -1;
+	}
 
 	g_free_not_null(dcc->file);
 	dcc->file = dcc_get_download_path(dcc->arg);
@@ -253,6 +256,12 @@ static void sig_dccget_connected(GET_DCC_REC *dcc)
 	dcc->tagread = g_input_add(dcc->handle, G_INPUT_READ,
 				   (GInputFunction) sig_dccget_receive, dcc);
 	signal_emit("dcc connected", 1, dcc);
+
+	if (dcc->from_dccserver) {
+		str = g_strdup_printf("121 %s %d\n",
+				      dcc->server ? dcc->server->nick : "??", 0);
+		net_transmit(dcc->handle, str, strlen(str));
+	}
 }
 
 void dcc_get_connect(GET_DCC_REC *dcc)
@@ -262,7 +271,13 @@ void dcc_get_connect(GET_DCC_REC *dcc)
 			DCC_GET_RENAME : DCC_GET_OVERWRITE;
 	}
 
+	if (dcc->from_dccserver) {
+		sig_dccget_connected(dcc);
+		return;
+	}
+
 	dcc->handle = dcc_connect_ip(&dcc->addr, dcc->port);
+
 	if (dcc->handle != NULL) {
 		dcc->tagconn =
 			g_input_add(dcc->handle,
@@ -276,6 +291,54 @@ void dcc_get_connect(GET_DCC_REC *dcc)
 	}
 }
 
+static void dcc_get_listen(GET_DCC_REC *dcc)
+{
+	GIOChannel *handle;
+	IPADDR addr;
+	int port;
+
+	/* accept connection */
+	handle = net_accept(dcc->handle, &addr, &port);
+	if (handle == NULL)
+		return;
+
+	net_disconnect(dcc->handle);
+	g_source_remove(dcc->tagconn);
+	dcc->tagconn = -1;
+
+	dcc->starttime = time(NULL);
+	dcc->handle = handle;
+	memcpy(&dcc->addr, &addr, sizeof(IPADDR));
+	net_ip2host(&dcc->addr, dcc->addrstr);
+	dcc->port = port;
+
+	dcc->tagconn = g_input_add(handle, G_INPUT_READ | G_INPUT_WRITE,
+				   (GInputFunction) sig_dccget_connected, dcc);
+}
+
+void dcc_get_passive(GET_DCC_REC *dcc)
+{
+	GIOChannel *handle;
+	IPADDR own_ip;
+	int port;
+	char host[MAX_IP_LEN];
+
+	handle = dcc_listen(net_sendbuffer_handle(dcc->server->handle),
+			    &own_ip, &port);
+	if (handle == NULL)
+		cmd_return_error(CMDERR_ERRNO);
+
+	dcc->handle = handle;
+	dcc->tagconn = g_input_add(dcc->handle, G_INPUT_READ,
+				   (GInputFunction) dcc_get_listen, dcc);
+
+	/* Let's send the reply to the other client! */
+	dcc_ip2str(&own_ip, host);
+	irc_send_cmdv(dcc->server,
+		      "PRIVMSG %s :\001DCC SEND %s %s %d %"PRIuUOFF_T" %d\001",
+		      dcc->nick, dcc->arg, host, port, dcc->size, dcc->pasv_id);
+}
+
 #define get_params_match(params, pos) \
 	((is_numeric(params[pos], '\0') || is_ipv6_address(params[pos])) && \
 	is_numeric(params[(pos)+1], '\0') && atol(params[(pos)+1]) < 65536 && \
@@ -312,20 +375,39 @@ int get_file_params_count(char **params, int paramcount)
         return best;
 }
 
+char *get_file_name(char **params, int fileparams)
+{
+	GString *out = g_string_new(params[0]);
+	char *ret;
+	int pos;
+
+	for (pos = 1; pos < fileparams; pos++) {
+		out = g_string_append(out, " ");
+		out = g_string_append(out, params[pos]);
+	}
+
+	ret = out->str;
+	g_string_free(out, FALSE);
+	return ret;
+}
+
 /* CTCP: DCC SEND */
 static void ctcp_msg_dcc_send(IRC_SERVER_REC *server, const char *data,
 			      const char *nick, const char *addr,
 			      const char *target, CHAT_DCC_REC *chat)
 {
 	GET_DCC_REC *dcc;
-        IPADDR ip;
-	const char *address;
-	char **params, *fname;
+	SEND_DCC_REC *temp_dcc;
+	IPADDR ip;
+	char *address, **params, *fname;
 	int paramcount, fileparams;
 	int port, len, quoted = FALSE;
-        long size;
+        uoff_t size;
+	int p_id = -1;
+	int passive = FALSE;
 
 	/* SEND <file name> <address> <port> <size> [...] */
+	/* SEND <file name> <address> 0 <size> <id> (DCC SEND passive protocol) */
 	params = g_strsplit(data, " ", -1);
 	paramcount = strarray_length(params);
 
@@ -338,13 +420,18 @@ static void ctcp_msg_dcc_send(IRC_SERVER_REC *server, const char *data,
 
 	fileparams = get_file_params_count(params, paramcount);
 
-	address = params[fileparams];
+	address = g_strdup(params[fileparams]);
 	dcc_str2ip(address, &ip);
 	port = atoi(params[fileparams+1]);
-	size = atol(params[fileparams+2]);
+	size = str_to_uofft(params[fileparams+2]);
 
-	params[fileparams] = NULL;
-        fname = g_strjoinv(" ", params);
+	/* If this DCC uses passive protocol then store the id for later use. */
+	if (paramcount == fileparams + 4) {
+		p_id = atoi(params[fileparams+3]);
+		passive = TRUE;
+	}
+
+	fname = get_file_name(params, fileparams);
 	g_strfreev(params);
 
         len = strlen(fname);
@@ -354,36 +441,76 @@ static void ctcp_msg_dcc_send(IRC_SERVER_REC *server, const char *data,
 		g_memmove(fname, fname+1, len);
                 quoted = TRUE;
 	}
+    
+	if (passive && port != 0) {
+		/* This is NOT a DCC SEND request! This is a reply to our
+		   passive request. We MUST check the IDs and then connect to
+		   the remote host. */
+
+		temp_dcc = DCC_SEND(dcc_find_request(DCC_SEND_TYPE, nick, fname));
+		if (temp_dcc != NULL && p_id == temp_dcc->pasv_id) {
+			temp_dcc->target = g_strdup(target);
+			temp_dcc->port = port;
+			temp_dcc->size = size;
+			temp_dcc->file_quoted = quoted;
+
+			memcpy(&temp_dcc->addr, &ip, sizeof(IPADDR));
+			if (temp_dcc->addr.family == AF_INET)
+				net_ip2host(&temp_dcc->addr, temp_dcc->addrstr);
+			else {
+				/* with IPv6, show it to us as it was sent */
+				strocpy(temp_dcc->addrstr, address,
+					sizeof(temp_dcc->addrstr));
+			}
+
+			/* This new signal is added to let us invoke
+			   dcc_send_connect() which is found in dcc-send.c */
+			signal_emit("dcc reply send pasv", 1, temp_dcc);
+			g_free(address);
+			g_free(fname);
+			return;
+		} else if (temp_dcc != NULL && p_id != temp_dcc->pasv_id) {
+			/* IDs don't match... remove the old DCC SEND and
+			   return */
+			dcc_destroy(DCC(temp_dcc));
+			g_free(address);
+			g_free(fname);
+			return;
+		}
+	}
 
 	dcc = DCC_GET(dcc_find_request(DCC_GET_TYPE, nick, fname));
-	if (dcc != NULL) {
-		/* same DCC request offered again, remove the old one */
-		dcc_destroy(DCC(dcc));
-	}
+	if (dcc != NULL)
+		dcc_destroy(DCC(dcc)); /* remove the old DCC */
 
 	dcc = dcc_get_create(server, chat, nick, fname);
 	dcc->target = g_strdup(target);
+
+	if (passive && port == 0)
+		dcc->pasv_id = p_id; /* Assign the ID to the DCC */
+    
 	memcpy(&dcc->addr, &ip, sizeof(ip));
 	if (dcc->addr.family == AF_INET)
 		net_ip2host(&dcc->addr, dcc->addrstr);
 	else {
 		/* with IPv6, show it to us as it was sent */
-		strncpy(dcc->addrstr, address, sizeof(dcc->addrstr)-1);
-		dcc->addrstr[sizeof(dcc->addrstr)-1] = '\0';
+		strocpy(dcc->addrstr, address, sizeof(dcc->addrstr));
 	}
 	dcc->port = port;
 	dcc->size = size;
-        dcc->file_quoted = quoted;
+	dcc->file_quoted = quoted;
 
 	signal_emit("dcc request", 2, dcc, addr);
 
-        g_free(fname);
+	g_free(address);
+	g_free(fname);
 }
 
 /* handle receiving DCC - GET/RESUME. */
-void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func)
+void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func,
+		     DCC_GET_FUNC pasv_accept_func)
 {
-        GET_DCC_REC *dcc;
+	GET_DCC_REC *dcc;
 	GSList *tmp, *next;
 	char *nick, *fname;
 	void *free_arg;
@@ -397,8 +524,12 @@ void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func)
 
 	if (*nick == '\0') {
 		dcc = DCC_GET(dcc_find_request_latest(DCC_GET_TYPE));
-		if (dcc != NULL)
-			accept_func(dcc);
+		if (dcc != NULL) {
+			if (!dcc_is_passive(dcc))
+				accept_func(dcc);
+			else
+				pasv_accept_func(dcc);
+		}
 		cmd_params_free(free_arg);
 		return;
 	}
@@ -408,11 +539,14 @@ void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func)
 		GET_DCC_REC *dcc = tmp->data;
 
 		next = tmp->next;
-		if (IS_DCC_GET(dcc) && dcc_is_waiting_user(dcc) &&
-		    g_strcasecmp(dcc->nick, nick) == 0 &&
+		if (IS_DCC_GET(dcc) && g_strcasecmp(dcc->nick, nick) == 0 &&
+		    (dcc_is_waiting_user(dcc) || dcc->from_dccserver) &&
 		    (*fname == '\0' || strcmp(dcc->arg, fname) == 0)) {
 			found = TRUE;
-			accept_func(dcc);
+			if (!dcc_is_passive(dcc))
+				accept_func(dcc);
+			else
+				pasv_accept_func(dcc);
 		}
 	}
 
@@ -425,7 +559,7 @@ void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func)
 /* SYNTAX: DCC GET [<nick> [<file>]] */
 static void cmd_dcc_get(const char *data)
 {
-        cmd_dcc_receive(data, dcc_get_connect);
+	cmd_dcc_receive(data, dcc_get_connect, dcc_get_passive);
 }
 
 static void read_settings(void)
