@@ -33,7 +33,6 @@
 GSList *proxy_listens;
 GSList *proxy_clients;
 
-static CLIENT_REC *ctcp_client;
 static GString *next_line;
 static int ignore_next;
 
@@ -44,16 +43,13 @@ static void remove_client(CLIENT_REC *rec)
 	proxy_clients = g_slist_remove(proxy_clients, rec);
 	rec->listen->clients = g_slist_remove(rec->listen->clients, rec);
 
-	if (ctcp_client == rec)
-                ctcp_client = NULL;
-
 	signal_emit("proxy client disconnected", 1, rec);
 	printtext(NULL, NULL, MSGLEVEL_CLIENTNOTICE,
 		  "Proxy: Client disconnected from %s", rec->host);
 
 	g_free(rec->proxy_address);
-	net_disconnect(rec->handle);
-	g_source_remove(rec->tag);
+	net_sendbuffer_destroy(rec->handle, TRUE);
+	g_source_remove(rec->recv_tag);
 	line_split_free(rec->buffer);
 	g_free_not_null(rec->nick);
 	g_free_not_null(rec->host);
@@ -139,6 +135,7 @@ static void handle_client_connect_cmd(CLIENT_REC *client,
 static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 			      const char *data)
 {
+	GSList *tmp;
 	if (!client->connected) {
 		handle_client_connect_cmd(client, cmd, args);
 		return;
@@ -170,14 +167,26 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 	if (strcmp(cmd, "PROXY") == 0) {
 		if (g_strcasecmp(args, "CTCP ON") == 0) {
                         /* client wants all ctcps */
-			ctcp_client = client;
-			proxy_outdata(client, ":%s NOTICE %s :You're now receiving all CTCPs sent to proxy\n",
-				      client->proxy_address, client->nick);
+			client->want_ctcp = 1;
+	                for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
+				CLIENT_REC *rec = tmp->data;
+				if ((g_strcasecmp(client->listen->ircnet,rec->listen->ircnet) == 0) &&
+					/* kludgy way to check if the clients aren't the same */
+					(client->recv_tag != rec->recv_tag)) {
+						if (rec->want_ctcp == 1)
+							proxy_outdata(rec, ":%s NOTICE %s :Another client is now receiving CTCPs sent to %s\n",
+			                                      rec->proxy_address, rec->nick, rec->listen->ircnet);
+						rec->want_ctcp = 0;
+		                }
+						                                      
+			}
+			proxy_outdata(client, ":%s NOTICE %s :You're now receiving CTCPs sent to %s\n",
+				      client->proxy_address, client->nick,client->listen->ircnet);
 		} else if (g_strcasecmp(args, "CTCP OFF") == 0) {
                         /* client wants proxy to handle all ctcps */
-			ctcp_client = NULL;
-			proxy_outdata(client, ":%s NOTICE %s :Proxy is now handling itself all CTCPs\n",
-				      client->proxy_address, client->nick);
+			client->want_ctcp = 0;
+			proxy_outdata(client, ":%s NOTICE %s :Proxy is now handling itself CTCPs sent to %s\n",
+				      client->proxy_address, client->nick, client->listen->ircnet);
 		}
 		return;
 	}
@@ -249,7 +258,8 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 
 		ignore_next = TRUE;
 		if (*msg != '\001' || msg[strlen(msg)-1] != '\001') {
-			signal_emit("message own_public", 4,
+	        	signal_emit(ischannel(*target) ?
+				    "message own_public" : "message own_private", 4,
 				    client->server, msg, target, target);
 		} else if (strncmp(msg+1, "ACTION ", 7) == 0) {
 			/* action */
@@ -271,6 +281,12 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
 		g_free(params);
 	} else if (strcmp(cmd, "PING") == 0) {
 		proxy_redirect_event(client, "ping", 1, NULL, TRUE);
+	} else if (strcmp(cmd, "AWAY") == 0) {
+		/* set the away reason */
+		if (args != NULL) {
+			g_free(client->server->away_reason);
+			client->server->away_reason = g_strdup(args);
+		}
 	}
 
 	irc_send_cmd(client->server, data);
@@ -284,7 +300,7 @@ static void sig_listen_client(CLIENT_REC *client)
 	g_return_if_fail(client != NULL);
 
 	while (g_slist_find(proxy_clients, client) != NULL) {
-		recvlen = net_receive(client->handle, tmpbuf, sizeof(tmpbuf));
+		recvlen = net_receive(client->handle->handle, tmpbuf, sizeof(tmpbuf));
 		ret = line_split(tmpbuf, recvlen, &str, &client->buffer);
 		if (ret == -1) {
 			/* connection lost */
@@ -311,6 +327,7 @@ static void sig_listen(LISTEN_REC *listen)
 {
 	CLIENT_REC *rec;
 	IPADDR ip;
+	NET_SENDBUF_REC *sendbuf;
         GIOChannel *handle;
 	char host[MAX_IP_LEN];
 	int port;
@@ -322,10 +339,10 @@ static void sig_listen(LISTEN_REC *listen)
 	if (handle == NULL)
 		return;
 	net_ip2host(&ip, host);
-
+	sendbuf = net_sendbuffer_create(handle, 0);
 	rec = g_new0(CLIENT_REC, 1);
 	rec->listen = listen;
-	rec->handle = handle;
+	rec->handle = sendbuf;
         rec->host = g_strdup(host);
 	if (strcmp(listen->ircnet, "*") == 0) {
 		rec->proxy_address = g_strdup("irc.proxy");
@@ -335,7 +352,7 @@ static void sig_listen(LISTEN_REC *listen)
 		rec->server = servers == NULL ? NULL :
 			IRC_SERVER(server_find_chatnet(listen->ircnet));
 	}
-	rec->tag = g_input_add(handle, G_INPUT_READ,
+	rec->recv_tag = g_input_add(handle, G_INPUT_READ,
 			       (GInputFunction) sig_listen_client, rec);
 
 	proxy_clients = g_slist_prepend(proxy_clients, rec);
@@ -357,6 +374,7 @@ static void sig_incoming(IRC_SERVER_REC *server, const char *line)
 static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 			     const char *nick, const char *address)
 {
+	GSList *tmp;
         void *client;
         const char *signal;
 	char *event, *args;
@@ -387,7 +405,7 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 		if (sscanf(signal+6, "%p", &client) == 1) {
 			/* send it to specific client only */
 			if (g_slist_find(proxy_clients, client) != NULL)
-				net_transmit(((CLIENT_REC *) client)->handle, next_line->str, next_line->len);
+				net_sendbuffer_send(((CLIENT_REC *) client)->handle, next_line->str, next_line->len);
 			g_free(event);
                         signal_stop();
 			return;
@@ -398,11 +416,17 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	    strstr(args, " :\001") != NULL &&
 	    strstr(args, " :\001ACTION") == NULL) {
 		/* CTCP - either answer ourself or forward it to one client */
-		if (ctcp_client != NULL) {
-                        /* one of the clients answers */
-			net_transmit(ctcp_client->handle,
-				     next_line->str, next_line->len);
-			signal_stop();
+		for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
+	        	CLIENT_REC *rec = tmp->data;
+		                        
+			if (rec->want_ctcp == 1) {
+                        	/* only CTCP for the chatnet where client is connected to will be forwarded */
+                        	if (strstr(rec->proxy_address, server->connrec->chatnet) != NULL) {
+					net_sendbuffer_send(rec->handle,
+							    next_line->str, next_line->len);
+					signal_stop();
+				}
+			}
 		}
 		g_free(event);
 		return;
@@ -512,6 +536,16 @@ static void sig_message_own_public(IRC_SERVER_REC *server, const char *msg,
 		proxy_outserver_all(server, "PRIVMSG %s :%s", target, msg);
 }
 
+static void sig_message_own_private(IRC_SERVER_REC *server, const char *msg,
+                                   const char *target, const char *origtarget)
+{
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (!ignore_next)
+		proxy_outserver_all(server, "PRIVMSG %s :%s", target, msg);
+}
+
 static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
                                    const char *target)
 {
@@ -641,8 +675,9 @@ void proxy_listen_init(void)
 	signal_add("server event", (SIGNAL_FUNC) sig_server_event);
 	signal_add("event connected", (SIGNAL_FUNC) event_connected);
 	signal_add("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
-	signal_add("event nick", (SIGNAL_FUNC) event_nick);
+	signal_add_first("event nick", (SIGNAL_FUNC) event_nick);
 	signal_add("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_add("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_add("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
 }
@@ -659,6 +694,7 @@ void proxy_listen_deinit(void)
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
 	signal_remove("event nick", (SIGNAL_FUNC) event_nick);
 	signal_remove("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_remove("message own_private", (SIGNAL_FUNC) sig_message_own_private);
 	signal_remove("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
 }
